# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/decorrelation',
                'doc_host': 'https://kanglcn.github.io',
                'git_url': 'https://github.com/kanglcn/decorrelation',
                'lib_path': 'decorrelation'},
  'syms': { 'decorrelation.cli.co': { 'decorrelation.cli.co.de_emperical_co_pc': ( 'CLI/co.html#de_emperical_co_pc',
                                                                                   'decorrelation/cli/co.py')},
            'decorrelation.cli.load': { 'decorrelation.cli.load._cor_pos_dem': ('CLI/load.html#_cor_pos_dem', 'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load._fetch_slc_par_date': ( 'CLI/load.html#_fetch_slc_par_date',
                                                                                        'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load._geo_width_nlines': ( 'CLI/load.html#_geo_width_nlines',
                                                                                      'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load._rdc_width_nlines': ( 'CLI/load.html#_rdc_width_nlines',
                                                                                      'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.de_load_gamma_flatten_rslc': ( 'CLI/load.html#de_load_gamma_flatten_rslc',
                                                                                               'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.de_load_gamma_lat_lon_hgt': ( 'CLI/load.html#de_load_gamma_lat_lon_hgt',
                                                                                              'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.de_load_gamma_look_vector': ( 'CLI/load.html#de_load_gamma_look_vector',
                                                                                              'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.de_load_gamma_metadata': ( 'CLI/load.html#de_load_gamma_metadata',
                                                                                           'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.de_load_gamma_range': ( 'CLI/load.html#de_load_gamma_range',
                                                                                        'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.read_gamma_image': ( 'CLI/load.html#read_gamma_image',
                                                                                     'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.read_gamma_plist': ( 'CLI/load.html#read_gamma_plist',
                                                                                     'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.write_gamma_image': ( 'CLI/load.html#write_gamma_image',
                                                                                      'decorrelation/cli/load.py'),
                                        'decorrelation.cli.load.write_gamma_plist': ( 'CLI/load.html#write_gamma_plist',
                                                                                      'decorrelation/cli/load.py')},
            'decorrelation.cli.pc': { 'decorrelation.cli.pc.console_de_pc2ras': ( 'CLI/pc.html#console_de_pc2ras',
                                                                                  'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.console_de_ras2pc': ( 'CLI/pc.html#console_de_ras2pc',
                                                                                  'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc2ras': ('CLI/pc.html#de_pc2ras', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_diff': ('CLI/pc.html#de_pc_diff', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_intersect': ('CLI/pc.html#de_pc_intersect', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_select_data': ( 'CLI/pc.html#de_pc_select_data',
                                                                                  'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_thres_pc': ('CLI/pc.html#de_pc_thres_pc', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_thres_ras': ('CLI/pc.html#de_pc_thres_ras', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_pc_union': ('CLI/pc.html#de_pc_union', 'decorrelation/cli/pc.py'),
                                      'decorrelation.cli.pc.de_ras2pc': ('CLI/pc.html#de_ras2pc', 'decorrelation/cli/pc.py')},
            'decorrelation.cli.pl': {'decorrelation.cli.pl.de_emi': ('CLI/pl.html#de_emi', 'decorrelation/cli/pl.py')},
            'decorrelation.cli.shp': { 'decorrelation.cli.shp.de_select_ds_can': ( 'CLI/shp.html#de_select_ds_can',
                                                                                   'decorrelation/cli/shp.py'),
                                       'decorrelation.cli.shp.de_shp_test': ('CLI/shp.html#de_shp_test', 'decorrelation/cli/shp.py')},
            'decorrelation.cli.utils.appconfig': { 'decorrelation.cli.utils.appconfig.get_config': ( 'CLI/utils/appconfig.html#get_config',
                                                                                                     'decorrelation/cli/utils/appconfig.py')},
            'decorrelation.cli.utils.chunk_size': { 'decorrelation.cli.utils.chunk_size.get_az_chunk_size_from_az_chunk_size': ( 'CLI/utils/chunk_size.html#get_az_chunk_size_from_az_chunk_size',
                                                                                                                                 'decorrelation/cli/utils/chunk_size.py'),
                                                    'decorrelation.cli.utils.chunk_size.get_az_chunk_size_from_n_pc_chunk': ( 'CLI/utils/chunk_size.html#get_az_chunk_size_from_n_pc_chunk',
                                                                                                                              'decorrelation/cli/utils/chunk_size.py'),
                                                    'decorrelation.cli.utils.chunk_size.get_pc_chunk_size_from_n_az_chunk': ( 'CLI/utils/chunk_size.html#get_pc_chunk_size_from_n_az_chunk',
                                                                                                                              'decorrelation/cli/utils/chunk_size.py'),
                                                    'decorrelation.cli.utils.chunk_size.get_pc_chunk_size_from_n_pc_chunk': ( 'CLI/utils/chunk_size.html#get_pc_chunk_size_from_n_pc_chunk',
                                                                                                                              'decorrelation/cli/utils/chunk_size.py'),
                                                    'decorrelation.cli.utils.chunk_size.get_pc_chunk_size_from_pc_chunk_size': ( 'CLI/utils/chunk_size.html#get_pc_chunk_size_from_pc_chunk_size',
                                                                                                                                 'decorrelation/cli/utils/chunk_size.py')},
            'decorrelation.cli.utils.dask': { 'decorrelation.cli.utils.dask._pad_chunks': ( 'CLI/utils/dask.html#_pad_chunks',
                                                                                            'decorrelation/cli/utils/dask.py'),
                                              'decorrelation.cli.utils.dask.get_cuda_cluster': ( 'CLI/utils/dask.html#get_cuda_cluster',
                                                                                                 'decorrelation/cli/utils/dask.py'),
                                              'decorrelation.cli.utils.dask.get_pc_chunk_size': ( 'CLI/utils/dask.html#get_pc_chunk_size',
                                                                                                  'decorrelation/cli/utils/dask.py'),
                                              'decorrelation.cli.utils.dask.pad_internal': ( 'CLI/utils/dask.html#pad_internal',
                                                                                             'decorrelation/cli/utils/dask.py')},
            'decorrelation.cli.utils.logging': { 'decorrelation.cli.utils.logging._get_random_string': ( 'CLI/utils/logging.html#_get_random_string',
                                                                                                         'decorrelation/cli/utils/logging.py'),
                                                 'decorrelation.cli.utils.logging.darr_info': ( 'CLI/utils/logging.html#darr_info',
                                                                                                'decorrelation/cli/utils/logging.py'),
                                                 'decorrelation.cli.utils.logging.get_logger': ( 'CLI/utils/logging.html#get_logger',
                                                                                                 'decorrelation/cli/utils/logging.py'),
                                                 'decorrelation.cli.utils.logging.log_args': ( 'CLI/utils/logging.html#log_args',
                                                                                               'decorrelation/cli/utils/logging.py'),
                                                 'decorrelation.cli.utils.logging.zarr_info': ( 'CLI/utils/logging.html#zarr_info',
                                                                                                'decorrelation/cli/utils/logging.py')},
            'decorrelation.co': { 'decorrelation.co.emperical_co': ('API/co.html#emperical_co', 'decorrelation/co.py'),
                                  'decorrelation.co.emperical_co_pc': ('API/co.html#emperical_co_pc', 'decorrelation/co.py'),
                                  'decorrelation.co.isPD': ('API/co.html#ispd', 'decorrelation/co.py'),
                                  'decorrelation.co.nearestPD': ('API/co.html#nearestpd', 'decorrelation/co.py'),
                                  'decorrelation.co.regularize_spectral': ('API/co.html#regularize_spectral', 'decorrelation/co.py')},
            'decorrelation.pc': { 'decorrelation.pc._ras_dims': ('API/pc.html#_ras_dims', 'decorrelation/pc.py'),
                                  'decorrelation.pc.pc2ras': ('API/pc.html#pc2ras', 'decorrelation/pc.py'),
                                  'decorrelation.pc.pc_diff': ('API/pc.html#pc_diff', 'decorrelation/pc.py'),
                                  'decorrelation.pc.pc_intersect': ('API/pc.html#pc_intersect', 'decorrelation/pc.py'),
                                  'decorrelation.pc.pc_union': ('API/pc.html#pc_union', 'decorrelation/pc.py')},
            'decorrelation.pl': { 'decorrelation.pl.emi': ('API/pl.html#emi', 'decorrelation/pl.py'),
                                  'decorrelation.pl.temp_coh': ('API/pl.html#temp_coh', 'decorrelation/pl.py')},
            'decorrelation.plot': {'decorrelation.plot.bg_alpha': ('API/plot.html#bg_alpha', 'decorrelation/plot.py')},
            'decorrelation.shp': {'decorrelation.shp.ks_test': ('API/shp.html#ks_test', 'decorrelation/shp.py')}}}
